find_package(PythonInterp 3 REQUIRED)

#
# Generates Python sources from the .proto files
#
# grpc_generate_python (<SRCS> <MODS> <DEST> [<ARGN>...])
#
#  SRCS - variable to define with autogenerated source files
#  MODS - variable to define with autogenerated module __init__.py files
#  DEST - directory where the source files will be created
#  ARGN - .proto files
#
function(GRPC_GENERATE_PYTHON SRCS MODS DEST)
  if(NOT ARGN)
    message(SEND_ERROR "Error: GRPC_GENERATE_PYTHON() called without any proto files")
    return()
  endif()

  get_filename_component(PROTOBUF_IMPORT_DIRS ${PROTOBUF_IMPORT_DIRS} ABSOLUTE)
  list(APPEND _grpc_include_path -I ${PROTOBUF_IMPORT_DIRS})

  set(${SRCS})
  set(${MODS})
  set(_all_paths)
  foreach(FIL ${ARGN})
    get_filename_component(FIL ${FIL} ABSOLUTE)
    file(GLOB_RECURSE _sub_dir RELATIVE ${PROTOBUF_IMPORT_DIRS} "${FIL}")
    get_filename_component(_sub_dir ${_sub_dir} PATH)
    string(REPLACE "/" ";" _path_parts "${_sub_dir}")
    set(_rebuilt_path ${DEST})
    foreach(_path_part ${_path_parts})
      set(_rebuilt_path "${_rebuilt_path}/${_path_part}")
      set(_all_paths ${_all_paths} ${_rebuilt_path})
    endforeach()
    string(REPLACE "${PROTOBUF_IMPORT_DIRS}/" "" REL_DIR ${FIL})
    get_filename_component(REL_DIR ${REL_DIR} DIRECTORY)
    if(REL_DIR)
      string(APPEND REL_DIR "/")
    endif()
    get_filename_component(FIL_WE ${FIL} NAME_WE)
    set(_dst_src "${DEST}/${REL_DIR}${FIL_WE}_pb2_grpc.py")

    list(APPEND ${SRCS} "${_dst_src}")

    add_custom_command(
      OUTPUT "${_dst_src}"
      COMMAND ${PYTHON_EXECUTABLE}
      ARGS -m grpc_tools.protoc --grpc_python_out ${DEST} ${_grpc_include_path} ${FIL}
      DEPENDS ${FIL}
      COMMENT "Running Python gRPC compiler on ${FIL}"
      VERBATIM )
  endforeach()

  list(REMOVE_DUPLICATES _all_paths)
  foreach(_module ${_all_paths})
    set(_module_init_py "${_module}/__init__.py")
    list(APPEND ${MODS} "${_module_init_py}")
    add_custom_command(
      OUTPUT "${_module_init_py}"
      COMMAND touch
      ARGS ${_module_init_py}
      DEPENDS 
      COMMENT "Make Python module ${_module_init_py}"
      VERBATIM )
  endforeach()

  set_source_files_properties(${${SRCS}} ${${MODS}} PROPERTIES GENERATED TRUE)
  set(${SRCS} ${${SRCS}} PARENT_SCOPE)
  set(${MODS} ${${MODS}} PARENT_SCOPE)
endfunction()
